package com.bocloud.mcs.collector.manageone;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.MonitorAuth;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.ims.enums.CloudProvider;
import com.bocloud.mcs.aspect.EnableCollector;
import com.bocloud.mcs.collector.BocloudCollector;
import com.bocloud.mcs.collector.manageone.core.ManageOneClient;
import com.bocloud.mcs.service.internal.CmsCloudVendorService;
import com.megatron.common.encrypt.AESEncryptor;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.utils.ListTool;
import io.prometheus.client.GaugeMetricFamily;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

import java.util.*;
import java.util.concurrent.ConcurrentHashMap;

/**
 * manageone collector
 *
 * @author zhangyf
 * @since 2019-10-12
 */
@Component
@Slf4j
public class ManageOneHostCollector extends BocloudCollector implements InitializingBean {

    private final CmsCloudVendorService cmsCloudVendorService;

    List<Long> indicatorIds;
    JSONObject indicators;

    private String prefix = "bocloud_fusioncloud_host_";

    @Value("${collector.manageone.rate.enable:false}")
    boolean rateEnable;
    @Value("${collector.manageone.rate.value:10}")
    double rateValue;

    @Autowired
    public ManageOneHostCollector(CmsCloudVendorService cmsCloudVendorService) {
        this.cmsCloudVendorService = cmsCloudVendorService;
    }

    @Async
    @Override
    @EnableCollector(value = "${collector.manageone.host.enable:true}")
    @Scheduled(initialDelay = 1000L, fixedRate = 60000)
    public void scheduledCollect() {
        Date now = new Date();
        log.info("collect manageone host metrics start...");
        GaugeMetricFamily collectTimeUse = new GaugeMetricFamily(prefix + "collect_time_use", "help",
                Collections.singletonList("type"));
        List<CloudVendor> vendors = cmpCloudVendorService.list(CloudProvider.MANAGEONE);
        Map<String, GaugeMetricFamily> map = new ConcurrentHashMap<>();
        for (CloudVendor vendor : vendors) {
            try {
                Boolean isSyncServer = JSONObject.parseObject(vendor.getAuthentication()).getBoolean("isSyncServer");
                if (isSyncServer == null || !isSyncServer) {
                    continue;
                }
                ManageOneClient client = new ManageOneClient(rateEnable, rateValue);
                try {
                    // 获取fusioncloud运维侧地址及认证信息
                    GeneralResult authResult = cmsCloudVendorService.detailVendorAuth(vendor.getId());
                    Assert.isTrue(authResult.isSuccess(), "get vendor operations auth failed , error:" + authResult.getMessage());
                    MonitorAuth monitorAuth = JSONObject.parseObject(JSONObject.toJSONString(authResult.getData()), MonitorAuth.class);
                    Assert.notNull(monitorAuth, "vendor [" + vendor.getName() + "] not found auth config info");
                    String password = new AESEncryptor().decrypt(monitorAuth.getPassword(), null);
                    client.auth(monitorAuth.getAddress(), monitorAuth.getUsername(), password);
                } catch (Exception e) {
                    log.error("auth error ", e);
                    continue;
                }
                List<JSONObject> objects = client.listPhysicalHosts();//宿主机集合
                if (ListTool.isEmpty(indicatorIds)) {
                    indicatorIds = client.listIndicatorIds(client.getHostObjTypeId()); // 支持的监控指标集合IndicatorIds
                }
                if (indicators == null || indicators.size() == 0) {
                    indicators = client.listIndicators(indicatorIds); // 指标的数字和描述信息
                    log.debug(indicators.toJSONString());
                }
                client.collectMetrics(vendor, objects, indicatorIds, indicators, client.getHostObjTypeId(), map, prefix);
            } catch (RuntimeException e) {
                log.error("invoke manageone host get metrics error,", e);
            }
        }
        List<MetricFamilySamples> mfs = new ArrayList<>(map.values());
        collectTimeUse.addMetric(Collections.singletonList("manageone_host"), (System.currentTimeMillis() - now.getTime()));
        mfs.add(collectTimeUse);
        super.getCache().put(TIME, new Date());
        super.getCache().put(DATA, mfs);
        log.info("collect manageone host metrics end! use " + (System.currentTimeMillis() - now.getTime()) / 1000 + "s");
    }

    @Override
    public void afterPropertiesSet() {
        register();
    }

}
