package com.bocloud.mcs.collector.tianyicloud;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.ims.enums.CloudProvider;
import com.bocloud.mcs.aspect.EnableCollector;
import com.bocloud.mcs.collector.BocloudCollector;
import com.bocloud.mcs.collector.tianyicloud.core.TianyiProvider;
import com.bocloud.mcs.common.Authentication;
import com.megatron.common.encrypt.DESEncryptor;
import io.prometheus.client.GaugeMetricFamily;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.*;

@Component
@Slf4j
public class TianyiCloudCollector extends BocloudCollector implements InitializingBean {

    private static final String PREFIX = "tianyi_vm_";

    @Value("${metrics.tianyi.batchSize:100}")
    private Integer batchSize = 100;

    /**
     * 缓存过期时间，30分钟
     *
     * @return
     */
    @Override
    public int getExpire() {
        return 1800000;
    }

    @Async
    @Override
    @EnableCollector(value = "${collector.tianyi.enable:true}")
    @Scheduled(initialDelay = 1000L, fixedRate = 120000L)
    public void scheduledCollect() {
        Date now = new Date();
        log.info("collect tianyi vm metrics start...");
        GaugeMetricFamily collectTimeUse = new GaugeMetricFamily(PREFIX + "collect_time_use", "help",
                Collections.singletonList("type"));
        List<MetricFamilySamples> mfs = new ArrayList<>();
        List<String> labels = Arrays.asList("vendor_id", "instanceId", "region", "instanceUuid");
        List<CloudVendor> vendors = cmpCloudVendorService.list(CloudProvider.CTSTACK);
        for (CloudVendor vendor : vendors) {
            if (StringUtils.isBlank(vendor.getAuthentication())) {
                continue;
            }
            Authentication authentication = JSONArray.parseObject(vendor.getAuthentication(), Authentication.class);
            String accessKey = new DESEncryptor().decrypt(authentication.getAccessKey().trim(), "BocloudCMP").trim();
            String secretKey = new DESEncryptor().decrypt(authentication.getSecretKey().trim(), "BocloudCMP").trim();
            String projectId = authentication.getProjectId();
            for (String region : authentication.getRegions()) {
                try {
                    TianyiProvider tianyiProvider = new TianyiProvider(accessKey, secretKey, projectId, region, authentication.getUrl());
                    List<String> instanceIds = tianyiProvider.instanceList();

                    if (instanceIds == null || instanceIds.isEmpty()) {
                        log.warn("获取到云主机列表为null, 云平台名称: {}, 可用区:{}", vendor.getName(), region);
                        continue;
                    }
                    // 分割instanceIds，控制每批获取监控的云主机数量
                    for (List<String> instanceIdsBatch : splitList(instanceIds, batchSize)) {
                        List<JSONObject> metric = tianyiProvider.getMetric(instanceIdsBatch);
                        for (JSONObject jsonObject : metric) { // 每个metric为一个vm的监控数据
                            List<JSONObject> itemList = JSONObject.parseArray(jsonObject.getJSONArray("itemList").toJSONString(), JSONObject.class);
                            for (JSONObject metricItem : itemList) {
                                //windows机器，磁盘监控数据返回值会包括.C: ,暂定把点后面的删除
                                String name = (PREFIX + metricItem.getString("itemName"));
                                if (name.contains(".")) {
                                    name = name.substring(0, name.indexOf("."));
                                }
                                GaugeMetricFamily gmf = new GaugeMetricFamily(name.replace("./", ""), "help", labels);
                                gmf.addMetric(Arrays.asList(vendor.getId() + "", jsonObject.getString("deviceUUID"), jsonObject.getString("regionID"), jsonObject.getString("deviceUUID")),
                                        Double.parseDouble(metricItem.getString("value")));
                                mfs.add(gmf);
                            }
                        }
                    }
                } catch (Exception e) {
                    log.error("获取云主机监控信息失败, 云平台名称: {}, 可用区:{}, 异常原因", vendor.getName(), region, e);
                }
            }
        }
        collectTimeUse.addMetric(Arrays.asList("tianyi"), (System.currentTimeMillis() - now.getTime()));
        mfs.add(collectTimeUse);
        super.getCache().put(TIME, new Date());
        super.getCache().put(DATA, mfs);
        log.info("collect tianyi  metrics end! use " + (System.currentTimeMillis() - now.getTime()) / 1000 + "s");
    }

    public static List<List<String>> splitList(List<String> originalList, int batchSize) {
        List<List<String>> batches = new ArrayList<>();

        for (int i = 0; i < originalList.size(); i += batchSize) {
            int end = Math.min(i + batchSize, originalList.size());
            List<String> batch = new ArrayList<>(originalList.subList(i, end));
            batches.add(batch);
        }

        return batches;
    }

}
