package com.bocloud.mcs.collector.tianyicloud.core;

import com.alibaba.fastjson.JSONObject;
import com.megatron.common.enums.HttpRequestMethod;
import com.megatron.common.utils.MapTools;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.lang3.time.DateFormatUtils;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
public class TianyiHeaderUtils {

    public static Map<String, Object> createHeaders(HttpRequestMethod method, Map<String, Object> headers, Map<String, Object> params,
                                                    String accessKey, String secretKey, String ctUserId) {
        Map<String, Object> newHeaders = new HashMap<>();
        Map<String, Object> headersOfAuth = new HashMap<>();
        try {
            if (!MapTools.isEmpty(headers)) {
                newHeaders.putAll(headers);
            }
            String hybridDate = DateFormatUtils.format(new Date(), "yyyyMMdd'T'HHmmss'Z'");
            headersOfAuth.put("hybrid-date", hybridDate);
            String requestId = UUID.randomUUID().toString().replace("-", "");
            headersOfAuth.put("ctyun-hybrid-request-id", requestId);
            String forSignature = forSignature(method, headersOfAuth, params);
            byte[] kdate = kdate(hybridDate, accessKey, secretKey);
            byte[] signatureBytes = hmacSHA256(forSignature.getBytes(StandardCharsets.UTF_8), kdate);
            String signature = new String(Base64.encodeBase64(signatureBytes), StandardCharsets.UTF_8);
            List<String> keyList = new ArrayList<>(headersOfAuth.keySet());
            keyList = keyList.stream().sorted(Comparator.comparing(String::toString)).collect(Collectors.toList());
            String headersValue = String.join(";", keyList);
            String authorization = accessKey + " Header=" + headersValue + " Signature=" + signature;

            newHeaders.put("Content-Type", "application/json");
            newHeaders.put("hybrid-date", hybridDate);
            newHeaders.put("ctyun-hybrid-request-id", requestId);
            newHeaders.put("Hybrid-Authorization", authorization);
            newHeaders.put("ctUserId", ctUserId);

        } catch (Exception e) {
            log.error("构造请求头出现异常", e);
        }
        return newHeaders;

    }

    private static String forSignature(HttpRequestMethod method, Map<String, Object> headersOfAuth, Map<String, Object> params) {
        List<String> keyList = new ArrayList<>(headersOfAuth.keySet());
        keyList = keyList.stream().sorted(Comparator.comparing(String::toString)).collect(Collectors.toList());
        StringBuilder headerBuffer = new StringBuilder();
        for (int i = 0; i < keyList.size(); i++) {
            String key = keyList.get(i);
            headerBuffer.append(key).append(':').append(headersOfAuth.get(key));
            if (i != keyList.size() - 1) {
                headerBuffer.append('\n');
            }
        }
        String campmocalHeader = headerBuffer.toString();

        String query = null;
        String body = null;
        String encodedValue;
        Object value;
        if (method == HttpRequestMethod.GET) {
            if (params != null && !params.isEmpty()) {
                keyList = new ArrayList<>(params.keySet());
                keyList = keyList.stream().sorted(Comparator.comparing(String::toString)).collect(Collectors.toList());
                // aa=1&bb=2
                StringBuilder queryBuffer = new StringBuilder();
                for (String key : keyList) {
                    if (!queryBuffer.isEmpty()) {
                        queryBuffer.append('&');
                    }
                    value = params.get(key);
                    if (value == null) {
                        value = "";
                    }
                    encodedValue = URLEncoder.encode(String.valueOf(value), StandardCharsets.UTF_8);
                    queryBuffer.append(key).append('=').append(encodedValue);
                }
                query = queryBuffer.toString();
            }
        } else if (method == HttpRequestMethod.POST) {
            if (params != null && !params.isEmpty()) {
                body = JSONObject.toJSONString(params);
                body = MessageDigestUtils.encrypt(body, MessageDigestUtils.SHA_256);
            }
        }

        String forSignature = campmocalHeader;
        if (null != query) {
            forSignature = forSignature + "\n" + query;
        }
        if (null != body) {
            forSignature = forSignature + "\n" + body;
        }
        return forSignature;
    }

    private static byte[] kdate(String hybridDate, String accessKey, String secretKey) throws Exception {
        byte[] ktime = hmacSHA256(hybridDate.getBytes(StandardCharsets.UTF_8), secretKey.getBytes(StandardCharsets.UTF_8));
        byte[] kAk = hmacSHA256(accessKey.getBytes(StandardCharsets.UTF_8), ktime);
        byte[] kdate = hmacSHA256(hybridDate.substring(0, 8).getBytes(StandardCharsets.UTF_8), kAk);
        return kdate;
    }

    /**
     * HmacSHA256算法,返回的结果始终是32位
     *
     * @param contentBytes 待加密的内容
     * @param secretBytes  密钥
     * @return 加密后的内容
     */
    private static byte[] hmacSHA256(byte[] contentBytes, byte[] secretBytes) throws Exception {
        Mac hmacSha256 = Mac.getInstance("HmacSHA256");
        hmacSha256.init(new SecretKeySpec(secretBytes, "HmacSHA256"));
        return hmacSha256.doFinal(contentBytes);
    }
}